@@ -6,7 +6,6 @@ import sys
 from importlib import import_module
 from inspect import isclass
 from pathlib import Path
-from pprint import pprint  # noqa: W0611 F401
 from string import Template
 
 import strictyaml
@@ -34,8 +33,6 @@ class Workflow(Component):
         self.source = source
         if source == '-':
             source = sys.stdin.read()
-        elif isdict(source):
-            pass  # TODO
         else:
             try:
                 params = os.environ.copy()
@@ -47,8 +44,6 @@ class Workflow(Component):
                 raise ConfigurationError(
                     f'Missing environment variable "{e.args[0]}" (needed by {source})')
 
-        # print(f'*** {source!r}')
-        # print(f'*** {self.source!r}')
         self.definition = self.expand(Box(strictyaml.load(source).data, box_dots=True))
 
     __MISSING__ = object()
@@ -65,9 +60,6 @@ class Workflow(Component):
         elif isdict(element):
             useref = element.pop('<use>', None)
             if useref:
-                # if useref in seen:
-                #     raise ConfigurationError(f'Circular reference detected: {useref}')
-
                 seen.add(useref)
                 try:
                     useref = type(element)(settings[useref].to_dict())
@@ -93,26 +85,20 @@ class Workflow(Component):
     def prepare(self):
         """Locate and configure components."""
         self.debug('Configuring workflow')
-        # pprint(self.app)
         for wname, workflow in self.app.workflows.items():
-            # print(f'workflow: {wname}')
             for nname, node in workflow.items():
                 if nname != 'settings':
                     node.workflow = wname
                     node.key = f'{wname}.{nname}'
-                    # print(f'  node: {node.key}')
                     self._configure_node(node)
                     node.setdefault('outbox', dict(default=None))
                     for msgbox in ('inbox', 'outbox'):
                         self._configure_pipes(node, msgbox)
         for wname, workflow in self.app.workflows.items():
-            # print(f'workflow: {wname}')
             for nname, node in workflow.items():
                 if nname != 'settings':
                     for msgbox in ('inbox', 'outbox'):
                         self._configure_connections(node, msgbox)
-        # pprint(self.app)
-        # import sys; sys.exit()
 
     def _configure_node(self, node):
         try:
@@ -136,8 +122,6 @@ class Workflow(Component):
                 node.outbox = 'pipekit.pipe:NullPipe'
 
     def _configure_pipes(self, node, msgbox):
-        # self.debug(f'  * {msgbox}: {node.key}')
-
         # Get default channels from class, if any.
         defaults = getattr(node.component, msgbox.upper(), dict())
         if defaults:
@@ -161,8 +145,6 @@ class Workflow(Component):
             if not spec:
                 spec = 'pipekit.pipe:QueuePipe'
             if isstr(spec):
-                # if spec == '<void>':
-                #     spec = 'pipekit.pipe:NullPipe'
                 if spec.startswith('<'):
                     break  # reference will be later resolved in _configure_connections()
 
@@ -178,13 +160,9 @@ class Workflow(Component):
                 channel.component = resolve(channel.component)
             channels[name] = channel
 
-        # print(' mb 2', channels)
         node[msgbox] = channels
-        # self.debug(f'  ** {msgbox}: {node.key}')
 
     def _configure_connections(self, node, msgbox):
-        # self.debug(f'  * {msgbox}: {node.key}')
-
         for name, channel in node[msgbox].items():
             if isstr(channel):
                 if not channel.startswith('<'):
@@ -195,10 +173,6 @@ class Workflow(Component):
                 peer_node, peer_channel = self.peer_node(spec, node)
                 peer_msgbox = 'outbox' if msgbox == 'inbox' else 'inbox'
                 node[msgbox][name] = PipeRef(peer_node, peer_msgbox, peer_channel)
-                # self.debug(f'  ** new PipeRef: {node.key} < '
-                #          f'{peer_node.key}.{peer_msgbox}:{peer_channel}')
-
-        # self.debug(f'  ** {msgbox}: {node.key}')
 
     def build(self):
         """Instantiate and wire up nodes and pipes."""
@@ -206,14 +180,11 @@ class Workflow(Component):
         self.debug('Instantiating workflow')
         self._node_backlog = dict()
         for wname, workflow in self.app.workflows.items():
-            # self.debug(wname)
             if wname not in self.nodes:
                 self.debug(f'Creating workflow {wname}')
                 settings = workflow.pop('settings', {})
                 self.nodes[wname] = SubWorkflow(id=wname, workflow=self, parent=self, **settings)
             for _, node in workflow.items():
-                # if _ == 's3-mount':
-                #     import pudb; pudb.set_trace()
                 self._node_backlog[node.key] = node
         for node in list(self._node_backlog.values()):
             try:
@@ -222,8 +193,6 @@ class Workflow(Component):
             except Exception:
                 self.exception(f'Error while instantiating node {node.key}')
                 raise
-        # pprint(self.app)
-        # import sys; sys.exit()
 
     RESERVED_SETTINGS = set(
             'id process scale inbox ifilters ofilters outbox leaf conditions'.split())
@@ -234,8 +203,6 @@ class Workflow(Component):
             return
 
         # Validate settings.
-        # self.debug(f'  node: {node.key}')
-        # self.debug(f'  settings: {node.get("settings", dict())}')
         settings = node.get('settings', {})
         reserved_settings = self.RESERVED_SETTINGS.intersection(set(settings.keys()))
         if reserved_settings:
@@ -252,28 +219,19 @@ class Workflow(Component):
 
         inbox = dict()
         for channel, pipe in node.inbox.items():
-            # self.debug(f'  inbox.{channel}: {pipe}')
             if hasattr(pipe, 'instance'):
-                # self.debug(f'  * instance exists: {pipe.instance}')
                 inbox[channel] = pipe.instance
                 raise Exception('node instance exists')
 
             if isinstance(pipe, PipeRef):
-                # TODO: forbid multiple use of pipes
-                # self.debug(f'  * connection {pipe}')
                 try:
                     node.inbox[channel] = pipe = Box(component=pipe, instance=pipe.resolve())
                 except KeyError:
                     raise ConfigurationError(f'Pipe not found: {pipe}')
 
-                # self.debug(f'  * connected {pipe}')
-            else:  # if 'component' in pipe:
-              try:  # noqa: E111
-                # self.debug(f'  * component type: {type(pipe.component)}')
+            else:
                 pipe.instance = self.make_component(
                     pipe.component, id=f'{node.key}.inbox.{channel}', **pipe.get('settings', {}))
-              except:  # noqa: E111 E722
-                import pudb; pudb.set_trace()  # noqa: E702
             inbox[channel] = pipe.instance
 
         ifilters = self._make_filters(node, 'ifilters')
@@ -311,22 +269,16 @@ class Workflow(Component):
                 f'Node {node.key} should be a subclass of Node or a callable, '
                 f'got {type(node.component)} instead')
 
-        # if node.endswith('verify-workspace'):
-        #     import pudb
-        #     pudb.set_trace()
         node.instance = self.make_component(node_class, **node_args)
-        # print(f'    instance: {node.instance}')
         del self._node_backlog[node.key]
 
     def _make_filters(self, node, type_):
         """Instantiate and return filters wrapped in a PriorityRegistry."""
         filters = node.get(type_, {})
-        # pprint(filters)
         for name, filter_ in filters.items():
             filter_.instance = self.make_component(
                 resolve(filter_.component), id=f'{node.key}.{type_}.{name}',
                 **filter_.get('settings', {}))
-            # pprint([name, filter_])
         return PriorityRegistry(dict((k, f.instance) for k, f in filters.items()))
 
     def peer_node(self, spec, dependent):
@@ -379,13 +331,6 @@ def resolve(spec):
     """Parse component textual spec and import and return corresponding object."""
     try:
         module, attr, *_ = spec.rsplit(':', 1) + [None]
-        # nameparts = module.split('.')
-        # if nameparts[0] == 'pipekit':
-        #     module = '.'.join(nameparts[1:])
-        #     parent = 'pipekit'
-        # else:
-        #     parent = 'components'
-        # module = import_module(f'{parent}.{module}')
         module = import_module(module)
         return getattr(module, attr)
 
